=begin
Vector2
	Vector2.left		Devuelve instancia con valores (-1, 0)
	Vector2.right		Devuelve instancia con valores ( 1, 0)
	Vector2.up			Devuelve instancia con valores ( 0,-1)
	Vector2.down		Devuelve instancia con valores ( 0, 1)
	Vector2.one			Devuelve instancia con valores ( 1, 1)
	Vector2.zero		Devuelve instancia con valores ( 0, 0)
	
	Vector2.Angle(from,to)						Devuelve ángulo, en radianes, entre un punto de referencia y otro.
	Vector2.ClampMagnitude(vector,maxLength)	Devuelve un vector con misma dirección, pero magnitud máxima de maxLength
	Vector2.Distance(a,b)						Devuelve la distancia entre punto a y punto b
	Vector2.Dot(a,b)							Producto escalar de dos vectores (a.x * b.x, a.y * b.y).
	Vector2.Lerp(a,b,t)							Interpolación linear entre vectores a y b a razón de t.
	Vector2.Max(a,b)							Retorna vector con componentes máximos de a y b.
	Vector2.Min(a,b)							Retorna vector con componentes mínimos de a y b.
	Vector2.MoveTowards(current,target,maxDelta) Similar a lerp, pero utiliza una distancia delta máxima.
	
	magnitude			Devuelve la magnitud del vector.
	angle				Devuelve el ángulo del vector.
	normalized			Devuelve una copia del vector, con magnitud 1.
	sqrMagnitude		Cuadrado de la magnitud. Más ligero de calcular que la magnitud.
	[n]					Imita comportamiento de un array, x si n=0, y si n=1
	[n]=v				Imita comportamiento de un array, x si n=0, y si n=1
	to_s				Valor del vector en una cadena de texto.
	
	Normalize			Normaliza el vector.
	Set(x,y)			Setea los valores x e y del vector.
	
	-this				Valor inverso del vector (-x,-y).
	this + other		Suma de vectores. nil en cualquier otro caso.
	this - other		Resta de vectores. nil en cualquier otro caso.
	this * other		Multiplicación de un vector por un número, o de componentes de vector. nil en cualquier otro caso.
	this / other		División de un vector por un número, o de componentes de vector. nil en cualquier otro caso.
	this % other		Aplica operación modulus a magnitud de vector, other debe ser un número.
	this == other		Retorna true si los vectores son aproximadamente iguales (se usa una diferencia mínima de 0.00001 en los componentes).
=end
# Requires OZMath

class Vector2
  @@inaccuracy = 0.00001
  
  attr_accessor :x
  attr_accessor :y
  
  def initialize(_x=0,_y=0)
    @x = _x
    @y = _y
  end
  
  # ============================================================================
  # Static properties
  # ============================================================================
  def self.left
    return Vector2.new(-1,0)
  end
  def self.right
    return Vector2.new(1,0)
  end
  def self.up
    return Vector2.new(0,-1)
  end
  def self.down
    return Vector2.new(0,1)
  end
  def self.one
    return Vector2.new(1,1)
  end
  def self.zero
    return Vector2.new(0,0)
  end
  
  # ============================================================================
  # Instance properties
  # ============================================================================
  def magnitude
    return Math.hypot(@x,@y)
  end
  def angle
    return Math.atan2(@y,@x)
  end
  def normalized
    m = magnitude # buffer magnitude
    return Vector2.new(@x/m, @y/m)
  end
  def sqrMagnitude
    return (@x*@x + @y*@y)
  end
  def [](n)
    return @x if n==0
    return @y if n==1
  end
  def []=(n,v)
    @x=v if n==0
    @y=v if n==1
  end
  def to_s
    return "(#{@x},#{@y})"
  end
  
  # ============================================================================
  # Static methods
  # ============================================================================
  def self.Angle(from,to)
    dx = to.x - from.x
    dy = to.y - from.x
    return Math.atan2(dy,dx)
  end
  def self.ClampMagnitude(vector, maxLength)
    m = vector.magnitude
    if (m > maxLength)
      return (vector/m)*maxLength
    else
      return vector
    end
  end
  def self.Distance(a,b)
    dx = (a.x - b.x).abs
    dy = (a.y - b.x).abs
    return Math.hypot(a,b)
  end
  def self.Dot(a,b)
    return a.x*b.x + a.y*b.y
  end
  def self.Lerp(a,b,t)
    _x = OZMath.lerp(t,a.x,b.x)
    _y = OZMath.lerp(t,a.y,b.y)
    return Vector2.new(_x,_y)
  end
  def self.Max(a,b)
    _x = a.x>b.x ? a.x : b.x
    _y = a.y>b.y ? a.y : b.y
    return Vector2.new(_x,_y)
  end
  def self.Min(a,b)
    _x = a.x<b.x ? a.x : b.x
    _y = a.y<b.y ? a.y : b.y
    return Vector2.new(_x,_y)
  end
  def self.MoveTowards(current, target, maxDelta)
    return current if (current==target)
    dx = target.x - current.x
    dy = target.y - current.y
    m = Math.hypot(dx,dy)
    a = Math.atan2(dy,dx)
    m = m < maxDelta ? m : maxDelta
    _x = m * cos(a)
    _y = m * sin(a)
    return Vector2.new(current.x + _x, current.y + _y)
  end
  
  # ============================================================================
  # Instance methods
  # ============================================================================
  def Normalize
    m = magnitude # buffer magnitude
    @x = @x/m
    @y = @y/m
  end
  def Set(_x,_y)
    @x = _x
    @y = _y
  end
  
  # ============================================================================
  # Operators
  # ============================================================================
  def -@
    return Vector2.new(-@x, -@y)
  end
  def +(other)
    case other.class
    when Vector2
      return Vector2.new(@x + other.x, @y + other.y)
    end
    return nil
  end 
  def -(other)
    case other.class
    when Vector2
      return Vector2.new(@x - other.x, @y - other.y)
    end
    return nil
  end
  def *(other)
    case other.class
    when Numeric
      return Vector2.new(@x * other, @y * other)
    when Vector2
      return Vector2.new(@x * other.x, @y * other.y)
    end
    return nil
  end
  def /(other)
    case other.class
    when Numeric
      return Vector2.new(@x / other, @y / other)
    when Vector2
      return Vector2.new(@x / other.x, @y / other.y)
    end
    return nil
  end
  def %(other)
    case other.class
    when Numeric
      m = magnitude
      return normalized * (m%other)
    end
    return nil
  end
  
  def ==(other)
    case other.class
    when Vector2
      d_x = (@x-other.x).abs
      d_y = (@y-other.y).abs
      return (d_x < @@inaccuracy && d_y < @@inaccuracy)
    end
    return false
  end
  
end